module Fixture.ExtendedCounter exposing
    ( Model
    , Msg
    , encodeMsg
    , extendedMsg
    , initModel
    , msgDecoder
    , msgFromCounter
    , update
    )

import Example.Counter as Counter
import Json.Decode
import Json.Encode


type alias Model =
    Counter.Model


type Msg
    = CounterMsg Counter.Msg
    | ResetCounter


extendedMsg : Msg
extendedMsg =
    ResetCounter


msgFromCounter : Counter.Msg -> Msg
msgFromCounter =
    CounterMsg


initModel : Model
initModel =
    Tuple.first (Counter.init ())


update : Msg -> Model -> Model
update msg model =
    case msg of
        CounterMsg counterMsg ->
            Tuple.first (Counter.update counterMsg model)

        ResetCounter ->
            initModel


encodeMsg : Msg -> Json.Encode.Value
encodeMsg msg =
    case msg of
        CounterMsg counterMsg ->
            Counter.encodeMsg counterMsg

        ResetCounter ->
            Json.Encode.object [ ( "ResetCounter", Json.Encode.null ) ]


msgDecoder : Json.Decode.Decoder Msg
msgDecoder =
    Json.Decode.oneOf
        [ Json.Decode.field "ResetCounter" (Json.Decode.null ResetCounter)
        , Json.Decode.map CounterMsg Counter.msgDecoder
        ]
