module Test.AMR exposing (suite)

import AMR.Main as AMR
import Example.Counter as Counter
import Expect exposing (Expectation)
import Fixture.ExtendedCounter as ExtendedCounter
import Fuzz exposing (Fuzzer)
import Help
import Json.Decode
import Json.Encode
import Test exposing (Test, test)


suite : Test
suite =
    Test.describe "AMR"
        [ encodeDecodeEquality
        , decodeUntilError
        ]


encodeDecodeEquality : Test
encodeDecodeEquality =
    Test.fuzz counterModelFuzzer
        "Encoding and decoding a value yields the same value"
        (\model ->
            AMR.encode model
                |> Json.Decode.decodeValue
                    (AMR.decoder (Help.replayUpdate Counter.update)
                        Counter.msgDecoder
                        0
                    )
                |> Expect.equal (Ok model)
        )


decodeUntilError : Test
decodeUntilError =
    Test.fuzz (Fuzz.map2 Tuple.pair counterMsgsFuzzer counterMsgsFuzzer)
        "Decoding is done until the first unrecognized message"
        (\( replayMsgs, skipMsgs ) ->
            let
                msgsBeforeChange =
                    List.append (List.map ExtendedCounter.msgFromCounter replayMsgs)
                        (ExtendedCounter.extendedMsg :: List.map ExtendedCounter.msgFromCounter skipMsgs)

                modelBeforeChange =
                    List.foldl
                        (Help.replayUpdate
                            (AMR.update (Help.updateReplay ExtendedCounter.update)
                                (always Cmd.none)
                                ExtendedCounter.encodeMsg
                            )
                        )
                        (AMR.initModel ExtendedCounter.initModel)
                        msgsBeforeChange

                modelAfterChange =
                    Tuple.first
                        (AMR.init (Help.replayUpdate Counter.update)
                            Counter.msgDecoder
                            (Just (Json.Encode.encode 0 (AMR.encode modelBeforeChange)))
                            (Counter.init ())
                        )

                modelFromReplayMsgs =
                    List.foldl
                        (Help.replayUpdate
                            (AMR.update Counter.update
                                (always Cmd.none)
                                Counter.encodeMsg
                            )
                        )
                        (AMR.initModel (Tuple.first (Counter.init ())))
                        replayMsgs
            in
            Expect.equal modelAfterChange modelFromReplayMsgs
        )


counterMsgsFuzzer : Fuzzer (List Counter.Msg)
counterMsgsFuzzer =
    Fuzz.list (Fuzz.map Counter.msgFromInt Fuzz.int)


counterModelFuzzer : Fuzzer (AMR.Model Counter.Model)
counterModelFuzzer =
    Fuzz.map
        (List.foldl
            (\n model ->
                Tuple.first
                    (AMR.update Counter.update
                        (always Cmd.none)
                        Counter.encodeMsg
                        (Counter.msgFromInt n)
                        model
                    )
            )
            (Tuple.first
                (AMR.init (Help.replayUpdate Counter.update)
                    Counter.msgDecoder
                    Nothing
                    (Counter.init ())
                )
            )
        )
        (Fuzz.list Fuzz.int)
